import React, { useState, useEffect, useRef, useMemo } from 'react'
import { Input, Button, Tree, Spin, Table, message } from 'antd'
import { CompassOutlined, DownOutlined } from '@ant-design/icons'
import styles from './index.less'
import {
  ArcGISSceneMap,
  AutoCompleteSearch,
  Drawtool as drawTool,
  Graphic,
  GraphicsLayer,
  Point,
  geomUtils,
  SimpleLineSymbol,
  SimpleFillSymbol,
} from '@wisdom-map/arcgismap'
import Drag from '../../../components/Drag'
import { isJson, isObject, getNanoid, mercatorToLngLat, getFieldInfo } from '../../../../utils'
import {
  FetchAreaList,
  GetAreaLayerTaskData,
  GetAccountConfigInfo,
  GetAccountPageList,
  GetCaseGISAreaData,
} from '../../../../apis/process'
import FileView from '../../../Account/components/TablePack/components/FileView'
import TagPack from '../../../components/TagPack'

const getRings = (rings) => {
  rings = rings.map(ring => {
    if (ring.length > 1) {
      if (ring[0][0] != ring[ring.length - 1][0] || ring[0][1] != ring[ring.length - 1][1]) {
        ring.push(ring[0])
      }
    }
    return ring
  })
  return rings
}

const AreaTask = (props) => {

  const codes = window?.pandaXform?.codes || { 工单编号: '', 事件编号: '', }
  const userID = window?.globalConfig?.userInfo?.OID || 1;
  const { token, client } = window.globalConfig
  const { value, onChange, schema, addons } = props
  const {
    titleShow,
    disabled,
    placeholder,
    presetValue,
    tableName,
    tableNameParent,
    addressSync,
    screenShot,
    service,
    layerName,
    areaTaskShine,
    accountName,
    fieldList,
  } = schema
  const [visible, setVisible] = useState(false)
  const [rings, setRings] = useState({})
  const [text, setText] = useState(presetValue ? '已绘制' : '（空）')
  const [put, setPut] = useState(true)
  const [tab, setTab] = useState('一级')
  const [areaList, setAreaList] = useState([])
  const [selectedKeys, setSelectedKeys] = useState([])
  const [loading, setLoading] = useState(false)
  const [config, setConfig] = useState({ accountFieids: [], formJson: '' })
  const { accountFieids, formJson } = config
  const [isGIS, setIsGIS] = useState(false)
  const [dataSource, setDataSource] = useState([])
  const [layerCount, setLayerCount] = useState([])
  const viewRef = useRef(null)

  const treeData = useMemo(() => {
    const ListToTree = (List, rootValue) => {
      const treeData = []
      if (List.length) {
        List.forEach(v => {
          if (v.parentID === rootValue) {
            const children = ListToTree(List, v.id)
            treeData.push({ key: v.id, title: v.name, parentID: v.parentID, areaPolygon: v.areaPolygon, children })
          }
        })
      }
      return treeData
    }
    const treeData = ListToTree(areaList, 0)
    if (tab === '二级') {
      let array = []
      treeData.forEach(v => {
        v.children.forEach(s => {
          array.push({ ...s })
        })
      })
      return array
    }
    if (tab === '三级') {
      let array = []
      treeData.forEach(v => {
        v.children.forEach(s => {
          s.children.forEach(j => {
            array.push({ ...j })
          })
        })
      })
      return array
    }
    return treeData
  }, [areaList, tab])

  const columns = useMemo(() => {

    const addOther = (v) => {
      array.push({
        title: v.fieldAliasName,
        dataIndex: isGIS ? v.fieldName : `${v.fieldType}-${v.fieldName}`,
        key: isGIS ? v.fieldName : `${v.fieldType}-${v.fieldName}`,
        render: (_, row) => {
          if (!row['台账-反馈状态']) return null
          let color = 'rgb(75, 172, 25)'
          if (row['台账-反馈状态'] === '进行中') {
            color = 'rgb(255, 169, 64)'
          }
          if (row['台账-反馈状态'] === '被驳回') {
            color = 'rgb(255, 77, 79)'
          }
          return <TagPack color={color} text={row['台账-反馈状态']} />
        }
      })
    }
    let array = []
    let widgetInfo = {}
    if (formJson) {
      widgetInfo = getFieldInfo(JSON.parse(formJson))
    }
    fieldList?.forEach(v => {
      let schema = widgetInfo[v.fieldName]
      if (v.fieldType === '台账' && schema?.widget === 'FileUpload') {
        array.push({
          title: v.fieldAliasName,
          dataIndex: isGIS ? v.fieldName : `${v.fieldType}-${v.fieldName}`,
          key: isGIS ? v.fieldName : `${v.fieldType}-${v.fieldName}`,
          render: (_, row) => {
            return <FileView value={row[`${v.fieldType}-${v.fieldName}`]} {...schema} />
          }
        })
      } else if (v.fieldType === '台账' && v.fieldName === '反馈状态') {
        //统一最后一行添加
      } else {
        array.push({
          title: v.fieldAliasName,
          dataIndex: isGIS ? v.fieldName : `${v.fieldType}-${v.fieldName}`,
          key: isGIS ? v.fieldName : `${v.fieldType}-${v.fieldName}`,
        })
      }
    })
    addOther({ fieldAliasName: '反馈状态', fieldType: '台账', fieldName: '反馈状态' })
    if (presetValue && !disabled) {
      array.push({
        title: '操作',
        dataIndex: '',
        key: '',
        width: 80,
        render: (_, row, index) => {
          if (row['台账-反馈状态'] === '已完成') {
            return <Button type='link' onClick={() => rowChange('被驳回', row)}>驳回</Button>
          }
          if (row['台账-反馈状态'] === '被驳回') {
            return <Button type='link' onClick={() => rowChange('已完成', row)}>取消</Button>
          }
        }
      })
    }
    return array
  }, [isGIS, fieldList, formJson, disabled, presetValue, dataSource])

  const onSelect = (value, areaList) => {
    setSelectedKeys(value)
    let areaPolygon = areaList.find(v => v.id === value?.[0])?.areaPolygon
    if (areaPolygon) {
      let geometry = geomUtils.toGeometry(JSON.parse(areaPolygon))
      setRings({ rings: geometry.rings })
      getGISDataCount({ rings: geometry.rings })
      let layer = new Graphic({
        geometry: geometry,
        symbol: new SimpleFillSymbol({
          color: [83, 164, 253, 0.5],
          outline: {
            color: [83, 164, 253, 1],
            width: 2,
          },
        }),
      })
      viewRef.current.goTo({ target: geometry, zoom: viewRef.current.zoom > 20 ? 20 : viewRef.current.zoom > 15 ? viewRef.current.zoom : 15 }, { duration: 500 })
      let graphicsLayer = viewRef.current.map.layers.find((lyr) => lyr.id == 'newDrawLayer')
      if (graphicsLayer) {
        graphicsLayer.removeAll()
      } else {
        graphicsLayer = new GraphicsLayer({ id: 'newDrawLayer' })
        viewRef.current.map.add(graphicsLayer)
      }
      graphicsLayer.add(layer)
    }
  }

  const createPaths = (view) => {
    setTimeout(() => {
      if (isJson(value)) {
        let layer = new Graphic({
          geometry: geomUtils.toGeometry(JSON.parse(value)),
          symbol: new SimpleFillSymbol({

          }),
        })
        let graphicsLayer = viewRef.current.map.layers.find((lyr) => lyr.id == 'newDrawLayer')
        if (graphicsLayer) {
          graphicsLayer.removeAll()
        } else {
          graphicsLayer = new GraphicsLayer({ id: 'newDrawLayer' })
          view.map.add(graphicsLayer)
        }
        graphicsLayer.add(layer)
      }
    }, 2000)
  }

  const rowChange = (type, row) => {
    let array = []
    let values = []
    dataSource.forEach(v => {
      array.push({ ...v, ['台账-反馈状态']: (v['台账-ID'] === row['台账-ID']) ? type : v['台账-反馈状态'] })
      if (v['台账-ID'] === row['台账-ID']) {
        values = [{ fieldName: '反馈状态', fieldValue: type }]
      }
    })
    let item = {
      type: 'edit',
      accountTable: accountName,
      ID: `${row['台账-ID']}`,
      values: values
    }
    let newData = []
    let oldData = window?.relationForm?.data || []
    if (!oldData.some(v => v.ID === row['台账-ID'])) {
      newData.push(item)
    }
    oldData.forEach(v => {
      if (v.ID === row['台账-ID']) {
        newData.push(item)
      } else {
        newData.push({ ...v })
      }
    })
    window.relationForm = { configs: [{ ...schema, '台账名称': schema.accountName, '映射字段': [{ fromField: '工单编号', toField: '工单编号' }] }], data: newData }
    setDataSource(array)
  }

  const getView = (viewObject) => {
    setRings(isJson(value) ? JSON.parse(value) : '')
    viewRef.current = viewObject
    createPaths(viewObject)
  }

  const getMapInfo = (viewObject) => {
    setRings(isJson(value) ? JSON.parse(value) : '')
    viewRef.current = viewObject
    createPaths(viewObject)
  }

  const getPoints = (view) => {
    if (view) {
      drawTool.activate({
        view,
        action: 'polygon',
        target: 'newDrawLayer',// 随便写
        toolTip: '左键选择位置',
        drawEnd: geometry => {
          setRings({ rings: geometry.rings })
          let layer = new Graphic({
            geometry: geometry,
            symbol: new SimpleFillSymbol({
              color: [83, 164, 253, 0.5],
              outline: {
                color: [83, 164, 253, 1],
                width: 2,
              },
            }),
          })
          let graphicsLayer = view.map.layers.find((lyr) => lyr.id == 'newDrawLayer')
          if (graphicsLayer) {
            graphicsLayer.removeAll()
          } else {
            graphicsLayer = new GraphicsLayer({ id: 'newDrawLayer' })
            view.map.add(graphicsLayer)
          }
          graphicsLayer.add(layer)
          drawTool.deactivate()
          getGISDataCount({ rings: getRings(geometry.rings) })
          setSelectedKeys([])
        },
        // rightClick: () => { getPoints(view, coordGetLayer) }
      })
    }
  }

  const start = () => {
    getPoints(viewRef.current)
  }

  const showMap = () => {
    if (addons) {
      getData(value)
      setVisible(true)
    }
  }

  const onCancel = () => {
    setVisible(false)
  }

  const getGISDataCount = async (rings) => {
    let layerWhere = []
    if (layerName?.length) {
      layerName.forEach(v => {
        layerWhere.push({ layerName: v, where: [] })
      })
    }
    let paramas = {
      mapServerName: service,
      geometry: JSON.stringify(rings),
      geometryType: 'civGeometryPolygon',
      layerWhereList: layerWhere
    }
    const { code, data } = await GetAreaLayerTaskData(paramas)
    if (code === 0) {
      let array = []
      if (layerName?.length) {
        layerName.forEach(v => {
          let count = data.filter(s => s['gis图层'] === v)?.length || 0
          array.push({ layerName: v, layerCount: count })
        })
      }
      setLayerCount(array)
    }
  }

  const getGISData = async () => {
    window.relationForm = { configs: [], data: [] }
    let layerWhere = []
    if (layerName?.length) {
      layerName.forEach(v => {
        layerWhere.push({ layerName: v, where: [] })
      })
    }
    let paramas = {
      mapServerName: service,
      geometry: JSON.stringify(rings),
      geometryType: 'civGeometryPolygon',
      layerWhereList: layerWhere
    }
    const { code, data, msg } = await GetAreaLayerTaskData(paramas)
    if (code === 0) {
      let items = []
      let array = []
      data.forEach(v => {
        let obj = {
          ID: `${getNanoid(10, '前端ID')}`
        }
        let values = []
        areaTaskShine.forEach(s => {
          if (s.fromField === '工单编号') {
            obj[s.toField] = codes['工单编号']
          } else if (s.fromField === 'GIS图层') {
            obj[s.toField] = v['gis图层'] || ''
          } else if (s.fromField === 'GIS编号') {
            obj[s.toField] = v['gis编号'] || ''
          } else {
            obj[s.toField] = v[s.fromField] || ''
          }
          values.push({ fieldName: s.toField, fieldValue: obj[s.toField] })
        })
        array.push({ ...v, ...obj })
        items.push({
          type: 'add',
          accountTable: accountName,
          ID: `${getNanoid(10, '前端ID')}`,
          values: values
        })
      })
      window.relationForm = { configs: [{ ...schema, '台账名称': schema.accountName, '映射字段': [{ fromField: '工单编号', toField: '工单编号' }] }], data: items }
      setDataSource(array)
      setIsGIS(true)
    } else {
      message.error(msg)
    }
  }

  const onOk = () => {
    if (selectedKeys.length) {
      onChange(`${selectedKeys[0]}`)
    } else {
      onChange(JSON.stringify(rings))
    }
    getGISData()
    setVisible(false)
  }

  const getData = async (value) => {
    const { code, data } = await FetchAreaList({ userID: userID, sourceType: '管网巡检,DMA分区' })
    if (code === '0') {
      setAreaList(data)
      if (value) {
        if (!isNaN(value)) {
          setTimeout(() => {
            onSelect([Number(value)], data)
          }, 2000)
        }
        if (isJson(value)) {
          getGISDataCount({ rings: getRings(JSON.parse(value).rings) })
        }
      }
    }
  }

  const getConfig = async () => {
    const { code, data } = await GetAccountConfigInfo(accountName)
    if (code === 0) {
      setConfig(data)
    }
  }

  const getDataSource = async () => {
    let params = {
      user: userID,
      mapServerName: service,
      caseNo: codes['工单编号'] || 'QB-2024-0000088',
      feedbackTableName: tableName,
      caseTableName: tableNameParent,
      // sortFields: '录入时间',
      // direction: 'desc',
      // timeField: '录入时间',
      // pageIndex: 0,
      // pageSize: 0,
      // queryWheres: [{ field: '工单编号', type: '等于', value: codes['工单编号'] }],
    }
    // if (!codes['工单编号']) return
    const { code, data, msg } = await GetCaseGISAreaData(params)
    if (code === 0) {
      setDataSource(data)
      setIsGIS(false)
    }
  }

  useEffect(() => {
    if (addons) {
      addons.setValueByPath(addons.dataPath, presetValue)
      if (presetValue) {
        getConfig()
        getDataSource()
      }
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (disabled && screenShot) {
      addons?.setSchemaByPath(addons.dataPath, { ...schema, width: '67%' })
    }
  }, [disabled, screenShot])

  return (
    <div className={styles.AreaTask}>
      <div className={styles.areaBox}>
        <div className={styles.areaHeader}> {titleShow} </div>
        <div className={styles.areaInput}>
          <div style={{ width: '96px', textAlign: 'right' }}>
            绘制选择：
          </div>
          {
            !disabled && !presetValue ? (
              <Input
                disabled={disabled}
                placeholder={disabled ? (placeholder || '') : (placeholder || '点击绘制区域')}
                value={isJson(value) ? '已绘制' : (value ? '已选择' : '')}
                addonAfter={disabled ? null : <CompassOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }} onClick={showMap} />}
                style={{ flex: 1 }}
                onClick={() => showMap()}
              />
            )
              :
              <Button
                icon={<CompassOutlined style={{ color: value ? '#0092fe' : 'rgba(0, 0, 0, 0.25)' }} />}
                onClick={() => {
                  if (value) {
                    showMap()
                  }
                }}
              >
                {value ? '查看区域' : '无区域信息'}
              </Button>
          }
        </div>
        <div className={styles.areaTable}>
          <Table
            size='small'
            rowKey={disabled ? 'gis-设备编号' : 'ID'}
            bordered
            columns={columns}
            dataSource={dataSource}
          />
        </div>
      </div>
      <Drag
        width={'80%'}
        title="选取区域"
        visible={visible}
        onOk={onOk}
        onCancel={onCancel}
        cancelText={'取消'}
        okText={'确定'}
        bodyStyle={{ height: 600, padding: '0' }}
        destroyOnClose={true}
        {...disabled ? { footer: null } : {}}
      >
        <div className={styles.AreaTaskMap}>
          <Spin spinning={loading} tip="获取GIS数据中...">
            <div className={styles.content}>
              <div className={styles.left}>
                {/* <div className={styles.tabs}>
                  {
                    ['一级', '二级', '三级'].map(v => <div className={styles.tab} active={`${v === tab}`} key={v} onClick={() => setTab(v)}>{v}</div>)
                  }
                </div> */}
                <div className={styles.tabContent}>
                  <Tree
                    defaultExpandAll
                    blockNode
                    selectedKeys={selectedKeys}
                    onSelect={(value) => onSelect(value, areaList)}
                    treeData={treeData}
                  />
                </div>
              </div>
              <div className={styles.right}>
                <div>
                  <ArcGISSceneMap
                    getMapInfo={getView}
                    widgets={[]}
                    token={token}
                    client={token ? client : 'sandbox'}
                  />
                </div>
                <div className={styles.description}>
                  <div>
                    <span>说明:</span>
                    <span style={{ marginLeft: '5px' }}>已选中区域内对象</span>
                  </div>
                  {
                    layerCount.map(v => {
                      return (
                        <div key={v.layerName}>
                          <span>{v.layerName}:</span>
                          <span style={{ marginLeft: '5px' }}>{v.layerCount}</span>
                        </div>
                      )
                    })
                  }
                </div>
                <Button
                  style={{ position: 'absolute', zIndex: '99', right: '10px', top: '10px' }}
                  type='primary'
                  onClick={start}
                >
                  开始绘制
                </Button>
              </div>
            </div>
          </Spin>
        </div>
      </Drag>
    </div>
  )

}

export default AreaTask